@@ -15,6 +15,17 @@
 
 using namespace KOSMIndoorMap;
 
+static QString formatDistance(int meter)
+{
+    if (meter < 1000) {
+        return i18n("%1m", meter);
+    }
+    if (meter < 10000) {
+        return i18n("%1km", ((int)meter/100)/10.0);
+    }
+    return i18n("%1km", (int)qRound(meter/1000.0));
+}
+
 bool OSMElementInformationModel::Info::operator<(OSMElementInformationModel::Info other) const
 {
     if (category == other.category) {
@@ -54,7 +65,7 @@ void OSMElementInformationModel::setElement(const OSMElement &element)
         reload();
     }
     endResetModel();
-    emit elementChanged();
+    Q_EMIT elementChanged();
 }
 
 void OSMElementInformationModel::clear()
@@ -66,7 +77,7 @@ void OSMElementInformationModel::clear()
     m_infos.clear();
     m_element = {};
     endResetModel();
-    emit elementChanged();
+    Q_EMIT elementChanged();
 }
 
 QString OSMElementInformationModel::name() const
@@ -165,7 +176,7 @@ struct {
     M("brand:wikipedia", Wikipedia, UnresolvedCategory),
     M("bus_routes", Routes, Main),
     M("buses", Routes, Main),
-    M("capacity", Capacity, Parking),
+    M("capacity", Capacity, UnresolvedCategory),
     M("capacity:charging", CapacityCharing, Parking),
     M("capacity:disabled", CapacityDisabled, Parking),
     M("capacity:parent", CapacityParent, Parking),
@@ -183,6 +194,9 @@ struct {
     M("email", Email, Contact),
     M("fee", Fee, UnresolvedCategory),
     M("maxstay", MaxStay, Parking),
+    M("mx:realtime_available", AvailableVehicles, Main),
+    M("mx:remaining_range", RemainingRange, Main),
+    M("mx:vehicle", Category, Header),
     M("network", Network, Operator),
     M("office", Category, Header),
     M("old_name", OldName, UnresolvedCategory),
@@ -293,9 +307,16 @@ void OSMElementInformationModel::resolveCategories()
                     info.category = Main;
                 }
                 break;
+            case Capacity:
+                if (m_element.tagValue("amenity").endsWith("rental")) {
+                    info.category = Main;
+                } else {
+                    info.category = Parking;
+                }
+                break;
             default:
             {
-                // for anything else: if it's not clearly something we have a secondary group for, resovle it to Main
+                // for anything else: if it's not clearly something we have a secondary group for, resolve it to Main
                 const auto amenity = m_element.tagValue("amenity");
                 if ((amenity != "parking" && amenity != "toilets")
                     || !m_element.tagValue("office").isEmpty()
@@ -411,6 +432,7 @@ QString OSMElementInformationModel::keyName(OSMElementInformationModel::Key key)
         case Takeaway: return i18n("Takeaway");
         case Socket: return i18nc("electrical power socket", "Socket");
         case OpeningHours: return {};
+        case AvailableVehicles: return i18n("Available vehicles");
         case Fee: return i18n("Fee");
         case Authentication: return i18n("Authentication");
         case BicycleParking: return i18n("Bicycle parking");
@@ -436,6 +458,7 @@ QString OSMElementInformationModel::keyName(OSMElementInformationModel::Key key)
         case OperatorName: return {};
         case Network: return i18nc("transport network", "Network");
         case OperatorWikipedia: return {};
+        case RemainingRange: return i18nc("remaining travel range of a battery powered vehicle", "Remaining range");
         case DebugLink: return QStringLiteral("OSM");
         case DebugKey: return {};
     }
@@ -455,6 +478,7 @@ QVariant OSMElementInformationModel::valueForKey(Info info) const
             l += m_element.tagValue("tourism").split(';');
             l += m_element.tagValue("vending").split(';');
             l += m_element.tagValue("office").split(';');
+            l += m_element.tagValue("mx:vehicle");
             if (l.isEmpty()) {
                 l += m_element.tagValue("room").split(';');
             }
@@ -547,6 +571,37 @@ QVariant OSMElementInformationModel::valueForKey(Info info) const
             return QLocale().createSeparatedList(l);
         }
         case OpeningHours: return QString::fromUtf8(m_element.tagValue("opening_hours"));
+        case AvailableVehicles:
+        {
+            const auto total = m_element.tagValue("mx:realtime_available").toInt();
+            QStringList types;
+            // there's no I18N_NOOP plural variants, so we have to use this more expensive approach
+            struct {
+                const char *tagName;
+                KLocalizedString msg;
+            } static const available_vehicles_map[] = {
+                { "mx:realtime_available:bike", ki18ncp("available rental vehicles", "%1 bike", "%1 bikes") },
+                { "mx:realtime_available:pedelec", ki18ncp("available rental vehicles", "%1 pedelec", "%1 pedelecs") },
+                { "mx:realtime_available:scooter", ki18ncp("available rental vehicles", "%1 kick scooter", "%1 kick scooters") },
+                { "mx:realtime_available:motorcycle", ki18ncp("available rental vehicles", "%1 moped", "%1 mopeds") },
+                { "mx:realtime_available:car", ki18ncp("available rental vehicles", "%1 car", "%1 cars") },
+            };
+            for (const auto &v : available_vehicles_map) {
+                const auto b = m_element.tagValue(v.tagName);
+                if (b.isEmpty()) {
+                    continue;
+                }
+                types.push_back(v.msg.subs(b.toInt()).toString());
+            }
+
+            if (types.isEmpty()) {
+                return QLocale().toString(total);
+            } else if (types.size() == 1) {
+                return types.at(0);
+            } else {
+                return i18n("%1 (%2)", total, QLocale().createSeparatedList(types));
+            }
+        }
         case Fee:
         {
             QByteArray fee;
@@ -637,6 +692,11 @@ QVariant OSMElementInformationModel::valueForKey(Info info) const
         case OperatorName: return QString::fromUtf8(m_element.tagValue("operator"));
         case Network: return QString::fromUtf8(m_element.tagValue("network"));
         case OperatorWikipedia: return wikipediaUrl(m_element.tagValue("operator:wikipedia", QLocale()));
+        case RemainingRange:
+        {
+            const auto range = m_element.tagValue("mx:remaining_range").toInt();
+            return formatDistance(range);
+        }
         case DebugLink: return m_element.url();
         case DebugKey: return {};
     }
@@ -726,8 +786,8 @@ QUrl OSMElementInformationModel::wikipediaUrl(const QByteArray &wp) const
 
     QUrl url;
     url.setScheme(QStringLiteral("https"));
-    url.setHost(s.leftRef(idx) + QLatin1String(".wikipedia.org"));
-    url.setPath(QLatin1String("/wiki/") + s.midRef(idx + 1));
+    url.setHost(QStringView(s).left(idx) + QLatin1String(".wikipedia.org"));
+    url.setPath(QLatin1String("/wiki/") + QStringView(s).mid(idx + 1));
     return url;
 }
 
